@@ -46,7 +46,7 @@ class ModelInfoBase(object):
         display text."""
 
         if hasattr(self, 'get_%s_display'%f_name):
-            ret = getattr(self, 'get_%s_display'%f_name)()
+            ret = getattr(self, 'get_%s_display'%(f_name.replace('.', '__')))()
         else:
             f = self.get_field(f_name)
 
@@ -62,24 +62,18 @@ class ModelInfoBase(object):
         f_value = None
 
         try:
-            return getattr(self, 'get_%s_value'%f_name)(instance)
+            return getattr(self, 'get_%s_value'%(f_name.replace('.', '__')))(instance)
         except AttributeError, e:
             pass
 
-        if isinstance(instance, models.Model):
-            try:
-                f_value = getattr(instance, f_name)
-            except ObjectDoesNotExist:
-                f_value = None
-        elif type(instance) == types.DictType:
-            f_value = instance.get(f_name, None)
+        try:
+            f_value = get_attr_value(instance, f_name)
+        except ObjectDoesNotExist:
+            f_value = None
 
         if f_value is None:
             return None
-
-        if callable(f_value):
-            return f_value()
-        
+    
         if isinstance(f_value, models.Model):
             if self._meta.auto_urlize and hasattr(f_value, 'get_absolute_url') and not f_name in self._meta.list_display_links:
                 return '<a href="%s">%s</a>'%(f_value.get_absolute_url(), unicode(f_value))
@@ -115,7 +109,7 @@ class ModelInfoBase(object):
                 if self._meta.auto_urlize: f_value = urlize(f_value)
                 if self._meta.auto_linebreaks: f_value = linebreaksbr(f_value)
 
-        return f_value
+        return f_value or ''
 
     def get_linkable_field_value(self, instance, f_name, f_value, force=False):
         url = ''
@@ -445,3 +439,37 @@ class ModelList(ModelInfoBase):
 
         return mark_safe(ret)
 
+# JUST COPIED FROM GERALDO REPORTS
+def get_attr_value(obj, attr_path):
+    """This function gets an attribute value from an object. If the attribute
+    is a method with no arguments (or arguments with default values) it calls
+    the method. If the expression string has a path to a child attribute, it
+    supports.
+    
+    Examples:
+        
+        attribute_name = 'name'
+        attribute_name = 'name.upper'
+        attribute_name = 'customer.name.lower'
+    """
+    if not attr_path:
+        raise Exception('Invalid attribute path \'%s\''%attr_path)
+
+    parts = attr_path.split('.')
+
+    try:
+        val = getattr(obj, parts[0])
+    except AttributeError:
+        try:
+            val = obj[parts[0]]
+        except (KeyError, TypeError):
+            raise Exception('There is no attribute nor key "%s" in the object "%s"'%(parts[0], repr(obj)))
+
+    if len(parts) > 1:
+        val = get_attr_value(val, '.'.join(parts[1:]))
+
+    if callable(val):
+        val = val()
+        
+    return val
+
